#include <iostream>
#include <fstream>
#include <string>
#include <sstream>
#include <vector>
#include <limits.h>

using namespace std;

int main(){
	// Soubory jsou otevirany jako file stream.
    ofstream outFile("1.txt");
    ifstream inFile, inFile2;
    // Soubor 3.txt je otevren jak pro zapis tak pro cteni zaroven.
    fstream ioFile("3.txt", ios::out | ios::in);

    // Pozadovany soubor lze otevrit i dodatecne
    inFile.open("2.txt");

    cout << hex << 12 << endl << "\n";
    // Rozhrani file streamu je stejne jako rozhrani objektu cout.
    // Je to proto, ze oba dedi rozhrani ze zakladni tridy ostream.
    outFile << hex << 12 << endl << "\n";

    int a;
    string text;
    // Preddefinovany objekt zdedeny z istream provadi automaticke
    // formatovani podle toho do jakeho typu promenne data nacita.
    // Zaroven automaticky oddeluje jednotlive tokeny. Napr. pokud
    // je mezi nimi vetsi mnozstvi mezer, tak je pri kazdem nacteni
    // preskoci az na dalsi token.

    cout << "--- Zakladni nacitani vstupu ---" << endl;
    cout << "Zadej slovo: ";
    cin >> text;
    cout << endl << "Zadano: " << text << endl;
    // Chteli jsme nacist pouze jedine slovo. Pokud jich uzivatel
	// zada vic, je mozne je ignorovat tim, ze je preskocime. Na
	// to pouzijeme metodu ignore(pocet_znaku_ktere_budou_ignorovany,
	// pripadny_ukoncovaci_znak). Pro prvni parametr muzeme pouzit
	// konstantu MAX_INT pro nejhorsi pripad a ukoncovaci znak muze
	// byt treba prechod na novy radek. Pri nasledujicim vstupu lze
	// zadat vice udaju, ale pouze prvni se pouzije a ostatni se zahodi.
    cout << "Zadej cislo: ";
    cin >> a;
    cin.ignore(INT_MAX, '\n');
    cout << endl << "Zadano: " << a << endl;
    // Nacitat lze i vice vstupu za sebou:
    cout << "Zadej cislo a slovo: ";
    cin >> a >> text;

    // Pokud nacitame napriklad cislo typu int a uzivatel
    // do terminalu vlozi misto toho text je to chyba.
    // Chybu detekujeme pomoci metody good():
    if(!cin.good()){
    	cout << "Spatne zadana data!" << endl;
    } else {
    	cout << "Zadano: " << a << " " << text << endl;
    }

    // Obcas se hodi mit rozhrani ostream na stringu. String
    // samotny neco takoveho bohuzel neumi, takze je treba jej
    // zabalit do objektu, ktery mu tutu funkcnost prida:
    stringstream stext(text);

    stext << a << " " << 12 << endl << hex << 12 << " " << oct << 12;
    cout << endl << "--- Nasledujici retezec byl vytvoren pomoci stringstreamu ---" << endl;
    cout << text << endl;

    // Jak jsme si jiz ukazali drive, tak jednotlive lexikalni
    // elementy (tokeny) lze oddelovat jednoduse pomoci rozhrani
    // istream. Bohuzel to funguje jen kdyz jsou oddeleny bilymi
    // znaky (Space, Tab, Enter). Pokud chceme nacist napr. cely
    // radek, nebo token oddeleny nejakym jinym oddelovacem, napr.
    // dvojteckou. Lze to provest pomoci getline.

    char buffer[100];

    // getline existuje ve dvou verzich - jako metoda objektu
    // istream, nebo jako samostatna funkce. Chovaji se podobne,
    // ale metoda pracuje se znakovym bufferem a funkce se stringem.

    // Naleduje kod, ktery do bufferu nacte 99 znaku nebo mene
    // pokud drive narazi na znak ':'

    cout << endl << "--- Pouziti metody getline ---" << endl;
    cout << "Zadej nejake znaky, dvojtecku a opet nejake znaky." << endl;
    cin.getline(buffer, 99, ':');
    cout << buffer << endl;

    // Nasleduje kod, ktery dostane nacita jednotlive tokeny oddelene
	// dvojteckou, dokud nedorazi do konce souboru. Jednotlive tokeny
	// jsou ukladany do stringu, ktera je ukladana do vektoru. Funkce
	// getline ma jako prvni argument jakykoliv objekt znedeny od istream,
	// lze tedy pouzit stringstream, ifstream, cin a dalsi.

	// Upozorneni 1:
	// Nasledujici kod funguje spravne na vstupu ze souboru. Pri vstupu
	// z konzole nema sanci poznat konec souboru, takze se mu musi
	// nasimulovat stiskem "Ctrl + d".

	// Upozorneni 2:
	// Nasledujici kod funguje spravne v linuxu, na solarisu se funkce
	// getline() chova spatne.

	cout << endl << "--- Pouziti funkce getline ---" << endl;
	cout << "Zadej slova oddelena dvouteckou. Zadavani ukoncis stiskem Enter a \"Ctrl+d\"." << endl;
    vector<string> data;
    while(!cin.eof()){
		getline(cin, text, ':');
		data.push_back(text);
    }
    // Vypiseme si oddelene casti textu kazdou na jeden radek.
    for(int i = 0; i < data.size(); ++i){
    	cout << data[i] << endl;
    }

    // Doted jsme se zabyvali formatovanym ctenim a zapisem. Alternativou
    // je neformatovany. Ke streamu je pristupovano jako k posloupnosti
    // bytu. Lze tedy zapisovat a cist bloky dat. Dale jsou implementovany
    // dva typy metod. Jedna na zjisteni aktualni pozice a druha na posun
    // v datech. Obe metody maji variantu s "g" a "p" na konci nazvu - put
    // a get. Slouzi na posun aktualni pozice v datovem streamu.

	cout << endl << "--- Ukazka neformatovaneho vystupu ---" << endl;
	outFile.write("123456789", 9);
	outFile.seekp(0, ios_base::beg);
	outFile.write("abcdef", 6);
	outFile.seekp(0, ios_base::beg);
	outFile.write("123", 3);
	cout << "Zapis do souboru skoncil na pozici " << outFile.tellp() << " a melo by v "
			<< "nem byt zapsano: 123def789." << endl;

	// Po skonceni prace se soubory je vhodne streamy zavrit.
	inFile.close();
	outFile.close();
	ioFile.close();

    // Metody formatovaneho a neformatovaneho IO lze libovolne kombinovat!

    return 0;
}
